@@ -2,160 +2,146 @@ package DTL::Fast;
 use strict; use warnings FATAL => 'all'; 
 use parent 'Exporter';
 use Carp;
-use Storable qw(freeze thaw);
+use Digest::MD5 qw(md5_hex);
 
 use 5.010;
-our $VERSION = '1.04';
+our $VERSION = '1.05';
+
+require XSLoader;
+XSLoader::load('DTL::Fast', $VERSION);
 
-use Cwd;
 use DTL::Fast::Template;
+use DTL::Fast::Cache::Runtime;
+use DTL::Fast::Cache::Serialized;
+
+our $RUNTIME_CACHE;
+our $SERIALIZED_CACHE;
 
 our @EXPORT_OK;
 
 # @todo Texts should be tossed by references
 # @todo These should be done via Cache class which can work with last modified date
-our %TEMPLATES_CACHE = ();
-our $TEMPLATES_CACHE_HITS = 0;
-our %OBJECTS_CACHE = ();
-our $OBJECTS_CACHE_HITS = 0;
 
 push @EXPORT_OK, 'get_template';
 sub get_template
 {
-    my $template_name = shift;
-    my $dirs = shift // [getcwd()];
-    my %kwargs = @_;
+    my( $template_name, %kwargs ) = @_;
     
     croak  "Template name was not specified" 
         if not $template_name;
-    
-    croak  "Second parameter must be a dirs array reference" 
-        if(
-            not ref $dirs
-            or ref $dirs ne 'ARRAY'
-            or not scalar @$dirs
-        );
 
-    my $cache_key = sprintf '%s:%s:%s:%s'
-        , __PACKAGE__
-        , $template_name
-        , join( ',', @$dirs )
-        , join( ',', @{$kwargs{'ssi_dirs'}//[]})
+    croak "Template directories array was not specified"
+        if
+            not defined $kwargs{'dirs'}
+            or ref $kwargs{'dirs'} ne 'ARRAY'
+            or not scalar @{$kwargs{'dirs'}}
         ;
 
-    my $template;
-    
-    if( exists $OBJECTS_CACHE{$cache_key} ) # already has object with this source parameters
-    {
-        $template = $OBJECTS_CACHE{$cache_key};
-        $OBJECTS_CACHE_HITS++;
-    }    
-    else
-    {
-        my $template_path;
+    my $cache_key = _get_cache_key( $template_name, %kwargs );
 
-        ($template, $template_path) = _read_template($template_name, $dirs, { '*path' => [] }, %kwargs );
+    my $template;
+
+    $RUNTIME_CACHE //= DTL::Fast::Cache::Runtime->new();
+    
+    if( 
+        $kwargs{'no_cache'}
+        or not defined ( $template = $RUNTIME_CACHE->get($cache_key))
+    )
+    {
+        $template = read_template($template_name, %kwargs );
         
         if( defined $template )
         {
-            $OBJECTS_CACHE{$cache_key} = $template;
+            $RUNTIME_CACHE->put($cache_key, $template);
         }
         else
         {
-            croak  sprintf( <<'_EOT_', $template_name, join("\n", @$dirs)) if not defined $template;
+            croak  sprintf( <<'_EOT_', $template_name, join("\n", @{$kwargs{'dirs'}}));
 Unable to find template %s in directories: 
 %s
 _EOT_
         }
-            
     }
-    
+   
     return $template;
 }
 
-sub _read_template
+sub _get_cache_key
 {
-    my $template_name = shift;
-    my $dirs = shift;
-    my $inheritance_path = shift // { '*path' => [] };
-    my %kwargs = @_;
+    my( $template_name, %kwargs ) = @_;
+    
+    return md5_hex(
+        sprintf( '%s:%s:%s:%s'
+            , __PACKAGE__
+            , $template_name
+            , join( ',', @{$kwargs{'dirs'}} )
+            , join( ',', @{$kwargs{'ssi_dirs'}//[]})
+            # shouldn't we pass uri_handler here?
+        )
+    )
+    ;
+}
+
+sub read_template
+{
+    my( $template_name, %kwargs ) = @_;
     
     my $template = undef;
     my $template_path = undef;
 
-    my $cache_key = sprintf '%s:%s:%s'
-        , __PACKAGE__
-        , $template_name
-        , join ',', @$dirs;
+    croak "Template directories array was not specified"
+        if not defined $kwargs{'dirs'}
+            or not ref $kwargs{'dirs'}
+            or not scalar @{$kwargs{'dirs'}}
+        ;
+    
+    my $cache_key = _get_cache_key( $template_name, %kwargs );
 
-    if( exists $TEMPLATES_CACHE{$cache_key} )   # already read template with this parameters
+    $SERIALIZED_CACHE //= DTL::Fast::Cache::Serialized->new();
+    
+    if( 
+        $kwargs{'no_cache'}
+        or not defined ( $template = $SERIALIZED_CACHE->get($cache_key))   # runtime serialized cache reading
+    )
     {
-        $template = thaw($TEMPLATES_CACHE{$cache_key});
-        $TEMPLATES_CACHE_HITS++;
-    }
-    else
-    {
-        ($template, $template_path) = _read_file($template_name, $dirs);
-        
-        if( defined $template )
+        if( 
+            $kwargs{'no_cache'}
+            or not exists $kwargs{'cache'} 
+            or not $kwargs{'cache'}
+            or not $kwargs{'cache'}->isa('DTL::Fast::Cache')
+            or not defined ($template = $kwargs{'cache'}->get($cache_key))
+        )
         {
-            if( exists $inheritance_path->{$template_path} )
-            {
-                croak  sprintf(
-                    "Recursive inheritance detected:\n%s\n" 
-                    , join "\n inherited from ", @{$inheritance_path->{'*path'}}, $template_path
-                );
-            }
-
-            my @arguments = (
-                $template
-                , $dirs
-                , 'file_path' => $template_path
-            );
-            push @arguments, 'ssi_dirs', $kwargs{'ssi_dirs'}
-                if $kwargs{'ssi_dirs'};
-            push @arguments, 'url_source', $kwargs{'url_source'}
-                if $kwargs{'url_source'};
+            ($template, $template_path) = _read_file($template_name, $kwargs{'dirs'});
             
-            $template = DTL::Fast::Template->new(@arguments);
-
-            if( $template->{'_extends' })  # template is inherited
+            if( defined $template )
             {
-                push @{$inheritance_path->{'*path'}}, $template_path;
-                $inheritance_path->{$template_path} = 1;
-
-                my ($parent_template, $parent_template_path) = _read_template(
-                    $template->{'_extends' }
-                    , $dirs
-                    , $inheritance_path
-                    , %kwargs
-                );
-                
-                if( defined $parent_template )
-                {
-                    $template = $parent_template->merge_blocks($template);
-                }
-                else
-                {
-                    croak  sprintf( "Couldn't found a parent template: %s in one of the following directories: %s"
-                        , $template->{'_extends' }
-                        , join( ', ', @$dirs)
-                    );
-                }
-                    
-                pop @{$inheritance_path->{'*path'}};
-                delete $inheritance_path->{$template_path};
+                $kwargs{'file_path'} = $template_path;
+                $template = DTL::Fast::Template->new( $template, %kwargs);
+        
+                $kwargs{'cache'}->put( $cache_key, $template )
+                    if 
+                        defined $template
+                        and exists $kwargs{'cache'}
+                        and $kwargs{'cache'}
+                        and $kwargs{'cache'}->isa('DTL::Fast::Cache')
+                    ;
             }
-            
-            $TEMPLATES_CACHE{$cache_key} = freeze($template);
         }
-            
+        
+        $SERIALIZED_CACHE->put($cache_key, $template)
+            if defined $template;
     }
     
-    return ($template, $template_path);
+    if( defined $template )
+    {
+        $template->{'cache'} = $kwargs{'cache'} if $kwargs{'cache'};
+        $template->{'url_source'} = $kwargs{'url_source'} if $kwargs{'url_source'};
+    }
+    
+    return $template;
 }
 
-
 sub _read_file
 {
     my $template_name = shift;
@@ -165,7 +151,7 @@ sub _read_file
     
     foreach my $dir (@$dirs)
     {
-        $dir =~ s/[\/\\]+$//gsi;
+        $dir =~ s/[\/\\]+$//xgsi;
         $template_path = sprintf '%s/%s', $dir, $template_name;
         if( 
             -e $template_path
@@ -173,10 +159,11 @@ sub _read_file
             and -r $template_path
         )
         {
-            if( open IF, '<', $template_path )
+            if( open my $IF, '<', $template_path )
             {
-                $template = join '', <IF>;
-                close IF;
+                local $/ = undef;
+                $template = <$IF>;
+                close $IF;
                 last;
             }
             else
@@ -197,8 +184,7 @@ sub _read_file
 push @EXPORT_OK, 'select_template';
 sub select_template
 {
-    my $template_names = shift;
-    my $dirs = shift // [getcwd()];
+    my( $template_names, %kwargs ) = @_;
     
     croak  "First parameter must be a template names array reference" 
         if(
@@ -211,7 +197,7 @@ sub select_template
     
     foreach my $template_name (@$template_names)
     {
-        if( ref ( $result = get_template( $template_name, $dirs )) eq 'DTL::Fast::Template' )
+        if( ref ( $result = get_template( $template_name, %kwargs )) eq 'DTL::Fast::Template' )
         {
             last;
         }
@@ -229,7 +215,7 @@ DTL::Fast - Perl implementation of Django templating language.
 
 =head1 VERSION
 
-Version 1.04
+Version 1.05
 
 =head1 SYNOPSIS
 
@@ -288,7 +274,7 @@ Using DTL::Fast::Template constructor:
     
     my $tpl = DTL::Fast::Template->new(
         $template_text,                             # template itself
-        [ $dir1, $dir2, ... ],                      # optional, directories list to look for parent templates and includes
+        'dirs' => [ $dir1, $dir2, ... ],            # optional, directories list to look for parent templates and includes
         'ssi_dirs' => [ $ssi_dir1, $ssi_dir1, ...]  # optional, directories list to look for files included with ssi tag
         'url_source' => \&uri_getter                # optional, reference to a function, that can return url template by model name (necessary for url tag)
     );
@@ -299,7 +285,7 @@ Using DTL::Fast::Template constructor:
     
     my $tpl = get_template(
         $template_path,                             # path to the template, relative to directories from second argument
-        [ $dir1, $dir2, ... ],                      # optional, directories list to look for parent templates and includes
+        'dirs' => [ $dir1, $dir2, ... ],            # mandatory, directories list to look for parent templates and includes
         'ssi_dirs' => [ $ssi_dir1, $ssi_dir1, ...]  # optional, directories list to look for files included with ssi tag
         'url_source' => \&uri_getter                # optional, reference to a function, that can return url template by model name (necessary for url tag)
     );
@@ -312,7 +298,7 @@ when you are using C<get_template> helper function, framework will try to find t
     
     my $tpl = select_template(
         [ $template_path1, $template_path2, ...],   # paths to templates, relative to directories from second argument
-        [ $dir1, $dir2, ... ],                      # optional, directories list to look for parent templates and includes
+        'dirs' => [ $dir1, $dir2, ... ],            # mandatory, directories list to look for parent templates and includes
         'ssi_dirs' => [ $ssi_dir1, $ssi_dir1, ...]  # optional, directories list to look for files included with ssi tag
         'url_source' => \&uri_getter                # optional, reference to a function, that can return url template by model name (necessary for url tag)
     );
@@ -327,7 +313,7 @@ After parsing template using one of the methods above, you may render it using c
     
     my $tpl = get_template(
         'hello_template.txt',          
-        [ '/srv/wwww/templates/' ]
+        'dirs' => [ '/srv/wwww/templates/' ]
     );
     
     print $tpl->render({ name => 'Alex' });
@@ -340,7 +326,7 @@ or
     
     my $tpl = get_template(
         'hello_template.txt',          
-        [ '/srv/wwww/templates/' ]
+        'dirs' => [ '/srv/wwww/templates/' ]
     );
     
     my $context = DTL::Fast::Context->new({
@@ -353,6 +339,19 @@ or
 
     $context->set('name' => 'Sergey');
     print $tpl->render($context);
+
+=head2 Cache classes
+
+To do...
+
+=head2 Custom tags
+
+To do...
+
+=head2 Custom filters
+
+To do...
+
     
 =head1 TEMPLATING LANGUAGE
 
@@ -436,6 +435,8 @@ is being rendered like:
 
 =over
 
+=item * C<as_bool>           - returns logical representation of object
+
 =item * C<and(operand)>      - makes logical `and` between object and operand
 
 =item * C<or(operand)>       - makes logical `or` between object and operand
@@ -509,11 +510,51 @@ This test rendered test template many times by external script, invoked via C<sy
     Fast render   : 62 wallclock secs ( 0.22 usr +  0.53 sys =  0.75 CPU) @ 668.45/s (n=500)
 
 Tests shows, that C<DTL::Fast> works 26% slower, than L<C<Dotiac::DTL>> in CGI environment.
-    
+
+=head2 DTL::Fast steps performance
+
+    1 Cache key  :  0 wallclock secs ( 0.19 usr +  0.00 sys =  0.19 CPU) @ 534759.36/s (n=100000)
+    2 Decompress :  0 wallclock secs ( 0.27 usr +  0.00 sys =  0.27 CPU) @ 377358.49/s (n=100000)
+    3 Serialize  :  4 wallclock secs ( 3.73 usr +  0.00 sys =  3.73 CPU) @ 26824.03/s (n=100000)
+    4 Deserialize:  5 wallclock secs ( 4.26 usr +  0.00 sys =  4.26 CPU) @ 23479.69/s (n=100000)
+    5 Compress   : 10 wallclock secs (10.50 usr +  0.00 sys = 10.50 CPU) @ 9524.72/s (n=100000)
+    6 Validate   : 11 wallclock secs ( 3.12 usr +  8.05 sys = 11.17 CPU) @ 8952.55/s (n=100000)
+
+    7 Parse      :  1 wallclock secs ( 0.44 usr +  0.23 sys =  0.67 CPU) @ 1492.54/s (n=1000)
+    8 Render     : 11 wallclock secs ( 9.30 usr +  1.14 sys = 10.44 CPU) @ 95.82/s (n=1000)    
+
 =head1 CHANGES
 
 =over
 
+=item * 18/01/2015 - v1.05 
+
+=over
+
+=item * Moved inheritance part into Template constructor 
+
+=item * Fixed bug with inheritance + blocks extension
+
+=item * Fixed bug with logic on arrays/hashes/scalars reference, object method as_bool support added. Tested.
+
+=item * Fixed bug with setting value to C<undef> using C<DTL::Fast::Context::set>
+
+=item * Fixed bug with inheritance path.
+
+=item * Implemented cache validation. Speed now is comparable to Dotiac::DTL.
+    
+=item * Made C<dirs> parameter optional for Template constructor, but it's still mandatory for C<get_template>/C<select_template>
+
+=item * Implemented cache classes: C<DTL::Fast::Cache>, C<DTL::Fast::Cache::Runtime>, C<DTL::Fast::Cache::Serialized>, C<DTL::Fast::Cache::Compressed>, C<DTL::Fast::Cache::File> and C<DTL::Fast::Cache::Memcached>. 
+
+=item * New dependencies added: L<C<Compress::Zlib>>, L<C<Digest::MD5>>
+
+=item * Added Perl::Critic testing, complies level 4.
+
+=item * Implemented C realization of spaceless tag.
+
+=back
+
 =item * 14/01/2015 - v1.04
 
 =over
@@ -527,7 +568,7 @@ Tests shows, that C<DTL::Fast> works 26% slower, than L<C<Dotiac::DTL>> in CGI e
 =item * Added Russian version of C<pluralize> filter:
 
     use DTL::Fast;
-    use DTL::Fast::Template::Filter::Ru::Pluralize; # this will override default pluralize with Russian version.
+    use DTL::Fast::Filter::Ru::Pluralize; # this will override default pluralize with Russian version.
 
 =item * Refactored strings backup and parametrized filters.
 
