@@ -6,8 +6,8 @@ use DTL::Fast::Utils qw(has_method);
 
 sub new
 {
-    my $proto = shift;
-    my $context = shift // {};
+    my( $proto, $context ) = @_;
+    $context //= {};
     
     croak  "Context should be a HASH reference"
         if ref $context ne 'HASH';
@@ -19,12 +19,11 @@ sub new
 
 sub get
 {
-    my $self = shift;
-    my $variable_path = shift;
+    my( $self, $variable_path ) = @_;
 
     if( ref $variable_path ne 'ARRAY' )    # suppose that raw variable invoked, period separated
     {
-        $variable_path = [split /\.+/, $variable_path];
+        $variable_path = [split /\.+/x, $variable_path];
     }
     else
     {
@@ -39,8 +38,10 @@ sub get
         $namespace->{$variable_name}
         : undef; 
 
-    $variable = $variable->($self) 
-        if ref $variable eq 'CODE';
+    while( ref $variable eq 'CODE' )
+    {
+        $variable = $variable->($self) 
+    }
         
     $variable = $self->traverse($variable, $variable_path)
         if 
@@ -53,9 +54,7 @@ sub get
 # tracing variable path
 sub traverse
 {
-    my $self = shift;
-    my $variable = shift;
-    my $path = shift;
+    my( $self, $variable, $path ) = @_;
 
     foreach my $step (@$path)
     {
@@ -66,7 +65,7 @@ sub traverse
         }
         elsif( 
             $current_type eq 'ARRAY'
-            and $step =~ /^\-?\d+$/
+            and $step =~ /^\-?\d+$/x
         )
         {
             $variable = $variable->[$step];
@@ -84,26 +83,26 @@ sub traverse
             );
         }        
     }
-        
-    $variable = $variable->($self) 
-        if ref $variable eq 'CODE';
+
+    while( ref $variable eq 'CODE' )
+    {
+        $variable = $variable->($self);
+    }
 
     return $variable;
 }
 
 sub set
 {
-    my $self = shift;
-    my @sets = @_;
+    my( $self, @sets ) = @_;
     
-    while( 
-        defined (my $key = shift @sets)
-        and defined (my $val = shift @sets)
-    )
+    while( scalar @sets > 1 )
     {
-        if( $key =~ /\./ )  # traversed set
+        my $key = shift @sets;
+        my $val = shift @sets;
+        if( $key =~ /\./x )  # traversed set
         {
-            my @key = split /\.+/, $key;
+            my @key = split /\.+/x, $key;
             my $variable_name = pop @key;
             my $variable = $self->get([@key]);
             
@@ -119,7 +118,7 @@ sub set
             }
             elsif( 
                 $variable_type eq 'ARRAY'
-                and $variable_name =~ /^\-?\d+$/
+                and $variable_name =~ /^\-?\d+$/x
             )
             {
                 $variable->[$variable_name] = $val;
@@ -140,17 +139,17 @@ sub set
     return $self;
 }
 
-sub push
+sub push_scope
 {
-    my $self = shift;
+    my( $self ) = @_;
 #    push @{$self->{'ns'}}, {}; # multi-level version, suppose it's slower on reading
     push @{$self->{'ns'}}, {%{$self->{'ns'}->[-1] // {}}};
     return $self;
 }
 
-sub pop
+sub pop_scope
 {
-    my $self = shift;
+    my( $self ) = @_;
     croak  "It's a last context layer available."
         if scalar @{$self->{'ns'}} ==  1;
     pop @{$self->{'ns'}};
