@@ -1,6 +1,7 @@
 package DTL::Fast::Parser;
 use strict; use warnings FATAL => 'all'; 
 use parent 'DTL::Fast::Renderer';
+use Carp;
 
 use DTL::Fast::Expression;
 use DTL::Fast::Text;
@@ -9,21 +10,22 @@ sub new
 {
     my( $proto, %kwargs ) = @_;
 
-    die 'No directory arrays passed into constructor'
+    croak 'No directory arrays passed into constructor'
         if not $kwargs{'dirs'}
             or ref $kwargs{'dirs'} ne 'ARRAY'
         ;
     
-    die 'No raw chunks array passed into constructor'
+    croak 'No raw chunks array passed into constructor'
         if not $kwargs{'raw_chunks'}
             or ref $kwargs{'raw_chunks'} ne 'ARRAY'
         ;
     
     $kwargs{'safe'} //= 0;
+    $kwargs{'blocks'} = {};
 
     my $self = $proto->SUPER::new(%kwargs)->parse_chunks();
     
-    delete @{$self}{'raw_chunks'};
+    delete @{$self}{'_template', '_container_block', 'raw_chunks'};
     
     return $self;
 }
@@ -44,31 +46,20 @@ sub parse_next_chunk
     my $chunk = shift @{$self->{'raw_chunks'}};
     
 #    warn "Processing chunk $chunk";
-    if( $chunk =~ /^\{\{\s*(.+?)\s*\}\}$/s )
+    if( $chunk =~ /^\{\{ (.+?) \}\}$/ )
     {
-        if( $1 eq 'block.super' )
-        {
-            require DTL::Fast::Tag::BlockSuper;
-            $chunk = DTL::Fast::Tag::BlockSuper->new(
-                ''
-                , 'dirs' => $self->{'dirs'}
-            );
-        }
-        else
-        {
-            $chunk = DTL::Fast::Variable->new($1);
-        }
+        $chunk = DTL::Fast::Variable->new($1);
     }
     elsif
     ( 
-        $chunk =~ /^\{\%\s*([^\s]+?)(?:\s+(.*?))?\s*\%\}$/s
+        $chunk =~ /^\{\% ([^\s]+?)(?: (.*?))? \%\}$/ 
     )
     {
-        $chunk = $self->parse_tag_chunk(lc $1, $2);
+        $chunk = $self->parse_tag_chunk($1, $2);
     }
     elsif
     ( 
-        $chunk =~ /^\{\#.*\#\}$/s 
+        $chunk =~ /^\{\# .* \#\}$/ 
     )
     {
         $chunk = undef;
@@ -91,35 +82,77 @@ sub parse_tag_chunk
     
     my $result = undef;
 
-    # dynamic module loading
-    if( 
-        not exists $DTL::Fast::TAG_HANDLERS{$tag_name} 
-        and exists $DTL::Fast::KNOWN_TAGS{$tag_name} 
-    )
-    {
-        require Module::Load;
-        Module::Load::load($DTL::Fast::KNOWN_TAGS{$tag_name});
-        $DTL::Fast::LOADED_MODULES{$DTL::Fast::KNOWN_TAGS{$tag_name}} = time;            
-    }
-
     if( exists $DTL::Fast::TAG_HANDLERS{$tag_name} )
     {     
         $result = $DTL::Fast::TAG_HANDLERS{$tag_name}->new(
             $tag_param
             , 'raw_chunks' => $self->{'raw_chunks'}
             , 'dirs' => $self->{'dirs'}
+            , '_template' => $self->{'_template'} // $self
+            , '_container_block' => $self->get_container_block()
         );
     }
     else
     {
-        warn sprintf ('Unknown tag (probably duplicated close tag): %s in %s'
-            , $tag_name // 'undef'
-            , $DTL::Fast::Template::CURRENT_TEMPLATE->{'file_path'}
-        );
+        warn "Unknown tag: $tag_name";
         $result = DTL::Fast::Text->new();
     }
     
     return $result;
 }
 
-1;
+sub get_container_block{ 
+    my( $self ) = @_;
+    return $self->{'_container_block'} 
+        // croak sprintf(
+            "There is no container block in: %s", $self // 'undef'
+        ); 
+}
+
+sub add_blocks
+{
+    my( $self, $blocks ) = @_;
+    
+    croak "Blocks must be a HASH reference" if ref $blocks ne 'HASH';
+    
+    foreach my $block_name (keys(%$blocks))
+    {
+        if( exists $self->{'blocks'}->{$block_name} )
+        {
+            croak "Block $block_name is already registered. Duplicate names are not allowed";
+        }
+        
+        $self->{'blocks'}->{$block_name} = $blocks->{$block_name};
+    }
+    
+    if( $self->{'_container'} )
+    {
+        $self->{'_container'}->add_blocks($blocks);
+    }    
+    
+    return $self;
+}
+
+sub remove_blocks
+{
+    my ($self, $block_names ) = @_;
+
+    croak "Blocks must be an ARRAY reference" if ref $block_names ne 'ARRAY';
+     
+    foreach my $block_name (@$block_names)
+    {
+        croak "Sub-block $block_name does not registered in current block."
+            if not exists $self->{'blocks'}->{$block_name};
+            
+        delete $self->{'blocks'}->{$block_name};
+    }
+  
+    if( $self->{'_container'} )
+    {
+        $self->{'_container'}->remove_blocks($block_names);
+    }    
+    
+    return $self;
+}
+
+1;
\ No newline at end of file
