@@ -1,6 +1,7 @@
 package DTL::Fast::Tag::Widthratio;
 use strict; use utf8; use warnings FATAL => 'all'; 
 use parent 'DTL::Fast::Tag::Simple';
+use Carp qw(confess);
 
 $DTL::Fast::TAG_HANDLERS{'widthratio'} = __PACKAGE__;
 
@@ -19,7 +20,7 @@ sub parse_parameters
         
         die sprintf(
             "Three arguments should be passed to widthratio: %s %s"
-            , $self->{'parameter'}  // 'undef'
+            , $self->{'parameter'} 
             , scalar @{$self->{'sources'}}
         ) if scalar @{$self->{'sources'}} != 3;
     }
